import re
from typing import Any
import pandas as pd


class RegexPatterns:
    """
    Contains all functions with regex patterns to extract data from the text files
    """

    def __init__(self) -> None:
        """
        Initialize the RegexPatterns class.
        """
        # Initialize the class
        # This class does not need any specific initialization
        pass

    def _guarantee_unicity(
        self, result: list[Any], fill: str | int | None = "Unknown"
    ) -> list[Any]:
        """
        Guarantee that the returned result is a list with just one element.

        Args:
            result (list): List of strings extracted with regex.
            fill (str | int): Value to fill if nothing is captured by the regex. Default is "Unknown".

        Returns:
            list: List with exactly one string.
        """
        # If nothing is found, it is set as "Unkown"
        if result == []:
            return [fill]

        # If more than one info is found, only the first is considered
        elif len(result) > 1:
            result = [result[0]]

        return result

    def get_modality(self, hand: list[str]) -> list[str]:
        """
        Get name of poker modality (for example, "Hold'em No Limit").

        Args:
            hand (list): List of texts from a specific hand.

        Returns:
            list: List with just one string, i.e. the name of the modality.
        """
        # Pattern to extract
        regex = r",\s+\S+\s+(.*?)\s+-\s+(?:Level|Match)"

        # Get the first content of a played hand
        target = hand[0]

        # Apply regex
        result = re.findall(regex, target)

        # Normalize output
        result = self._guarantee_unicity(result)

        # Return list with exactly one element
        return result

    def get_tourn_id(self, hand: list[str]) -> list[str]:
        """
        Get ID of a specific tournament (for example, "3285336035").

        Args:
            hand (list): List of texts from a specific hand.

        Returns:
            list: List with just one string, i.e. the ID of the tournament.
        """
        # Pattern to extract
        regex = r"Tournament #(\d+),.*"

        # Get the first content of a played hand
        target = hand[0]

        # Apply regex
        result = re.findall(regex, target)

        # Normalize output
        result = self._guarantee_unicity(result)

        # Return list with exactly one element
        return result

    def get_hand_id(self, hand: list[str]) -> list[str]:
        """
        Get ID of a specific hand (for example, "230689290677").

        Args:
            hand (list): List of texts from a specific hand.

        Returns:
            list: List with just one string, i.e. the ID of the hand.
        """
        # Pattern to extract
        regex = r"Hand #(\d+):.*"

        # Get the first content of a played hand
        target = hand[0]

        # Apply regex
        result = re.findall(regex, target)

        # Normalize output
        result = self._guarantee_unicity(result)

        return result

    def get_players(self, hand: list[str]) -> list[str]:
        """
        Get list of active players in a specific hand.

        Args:
            hand (list): List of texts from a specific hand.

        Returns:
            list: List with the names of the active players.
        """
        # Pattern to extract
        regex = r"\nSeat \d{1}: (.*) \(\S+ in chips"

        # Get the first content of a played hand
        target = hand[0]

        # Apply regex
        players = re.findall(regex, target)

        # Normalize output
        players = [re.escape(x) for x in players]

        return players

    def get_number_of_active_players(self, hand: list[str]) -> list[int]:
        """
        Get the number of active players in a specific hand.

        Args:
            hand (list): List of texts from a specific hand.

        Returns:
            list: List with one integer element, the number of players.
        """
        # Pattern to extract
        regex = r"\nSeat \d{1}: (.*) \(.*"

        # Get the first content of a played hand
        target = hand[0]

        # Apply regex and count the items found
        n = len(re.findall(regex, target))

        # Normalize output
        result = [n]

        return result

    def get_buyin(self, hand: list[str]) -> list[str]:
        """
        Get buy-in paid.

        Args:
            hand (list): List of texts from a specific hand.

        Returns:
            list: List with the value of the buy-in paid.
        """
        # Pattern to extract
        regex = r"\d+, (\S+) .*"

        # Get the first content of a played hand
        target = hand[0]

        # Apply regex
        buyin = re.findall(regex, target)

        # Normalize output
        buyin = self._guarantee_unicity(buyin)

        return buyin

    def get_level(self, hand: list[str]) -> list[str]:
        """
        Get level of the tournament.

        Args:
            hand (list): List of texts from a specific hand.

        Returns:
            list: List with the level of the tournament (for example, ["IV"]).
        """
        # Pattern to extract
        regex = r" Level (\S+) .*"

        # Get the first content of a played hand
        target = hand[0]

        # Apply regex
        result = re.findall(regex, target)

        return result

    def get_blinds(self, hand: list[str]) -> list[list[float]]:
        """
        Get current blinds of the tournament.

        Args:
            hand (list): List of texts from a specific hand.

        Returns:
            list: List with the list of blinds (for example, [['10', '20']]).
        """
        # Pattern to extract
        regex = r" Level \w+ \((\d+)/(\d+)\).*"

        # Get the first content of a played hand
        target = hand[0]

        # Apply regex
        result = [[float(x) for x in re.findall(regex, target)[0]]]

        return result

    def get_ante(self, hand: list[str]) -> list[float | None]:
        """
        Get current ante of the tournament.

        Args:
            hand (list): List of texts from a specific hand.

        Returns:
            list: List with the current ante of the tournament (for example, [10]).
        """
        # Pattern to extract
        regex = r"posts the ante (\d+)"

        # Get the first content of a played hand
        target = hand[0]

        # Apply regex
        result = re.findall(regex, target)

        # Convert to integer
        result = [float(x) for x in result if x.isdigit()]

        # This list cannot be empty
        if result == []:
            return [None]

        # Get the maximum value and return in a list
        result = [max(result)]

        return result

    def get_time(self, hand: list[str]) -> list[pd.Timestamp]:
        """
        Get current datetime of the tournament.

        Args:
            hand (list): List of texts from a specific hand.

        Returns:
            list: List with the current datetime (for example, ['2020-11-06 10:02:19']).
        """
        # Pattern to extract
        regex = r"(\d{4}/\d{2}/\d{2} \d{1,2}:\d{1,2}:\d{1,2})"

        # Get the first content of a played hand
        target = hand[0]

        # Apply regex
        result = re.findall(regex, target)

        # Normalize string
        result = [pd.to_datetime(x) for x in result]

        # Normalize output
        result = self._guarantee_unicity(result)

        return result

    def get_table_size(self, hand: list[str]) -> list[int]:
        """
        Get total number of players per table in the tournament.

        Args:
            hand (list): List of texts from a specific hand.

        Returns:
            list: List with the number of players per table (for example, ['9']).
        """
        # Pattern to extract
        regex = r"(\d)-max"

        # Get the first content of a played hand
        target = hand[0]

        # Apply regex
        result = [int(re.findall(regex, target)[0])]

        return result

    def get_table_id(self, hand: list[str]) -> list[str]:
        """
        Get ID of a specific hand (for example, "1").

        Args:
            hand (list): List of texts from a specific hand.

        Returns:
            list: List with just one string, i.e. the ID of the table.
        """
        # Pattern to extract
        regex = r"Table \'\d+ (\d+)\' .*"

        # Get the first content of a played hand
        target = hand[0]

        # Apply regex
        result = re.findall(regex, target)

        return result

    def get_owner_cards(self, hand: list[str]) -> list[str]:
        """
        Get cards from the owner of the logs (for example, ['As', '5s']).

        Args:
            hand (list): List of texts from a specific hand.

        Returns:
            list: List with a list of the owners' cards, like [['As', '5s']].
        """
        # Pattern to extract
        regex = r"Dealt to .* \[(\S+) (\S+)\]"

        # Get the first content of a played hand
        target = hand[1]

        # Apply regex
        result = re.findall(regex, target)

        return result

    def get_owner(self, hand: list[str]) -> list[str]:
        """
        Get name of the owner of the logs (for example, 'garciamurilo').

        Args:
            hand (list): List of texts from a specific hand.

        Returns:
            list: List with the name of the owner, like ['garciamurilo'].
        """
        # Pattern to extract
        regex = r"Dealt to (.*) \[.*"

        # Get the first content of a played hand
        target = hand[1]

        # Apply regex
        result = re.findall(regex, target)

        return result

    def get_stack(self, player: str, hand: list[str]) -> list[float | None]:
        """
        Get the current stack of the player (for example, '500').

        Args:
            player (str): Name of the player.
            hand (list): List of texts from a specific hand.

        Returns:
            list: List with the stack of the player (for example, ['500']).
        """
        # Pattern to extract
        regex = rf"Seat \d+: {player} \((\d+) in chips"

        # Get the first content of a played hand
        target = hand[0]

        # Apply regex
        result = re.findall(regex, target)

        # Normalize output
        result = [float(x) for x in result if x.isdigit()]
        result = self._guarantee_unicity(result, fill=None)

        return result

    def get_posted_blind(self, player: str, hand: list[str]) -> list[float | None]:
        """
        Get blind posted by the player (for example, '30').

        Args:
            player (str): Name of the player.
            hand (list): List of texts from a specific hand.

        Returns:
            list: List with the blide posted by the player (for example, ['30']).
        """
        # Pattern to extract
        regex = rf"{player}: posts \w+ blind (\d+)"

        # Get the first content of a played hand
        target = hand[0]

        # Apply regex
        result = re.findall(regex, target)

        # Normalize output
        result = [float(x) for x in result if x.isdigit()]
        result = self._guarantee_unicity(result, fill=None)

        return result

    def get_posted_ante(self, player: str, hand: list[str]) -> list[float | None]:
        """
        Get ante posted by the player (for example, '10').

        Args:
            player (str): Name of the player.
            hand (list): List of texts from a specific hand.

        Returns:
            list: List with the ante posted by the player (for example, ['10']).
        """
        # Pattern to extract
        regex = rf"{player}: posts the ante (\d+)"

        # Get the first content of a played hand
        target = hand[0]

        # Apply regex
        result = re.findall(regex, target)

        # Normalize output
        result = [float(x) for x in result if x.isdigit()]
        result = self._guarantee_unicity(result, fill=None)

        return result

    def get_actions(
        self, player: str, hand: list[str], stage: str
    ) -> list[list[tuple[str, str]]]:
        """
        Get action the player (for example, ('call', '50')).

        Args:
            player (str): Name of the player.
            hand (list): List of texts from a specific hand.
            stage (str): Name of the stage to be considered

        Returns:
            list: List of actions in a stage, like [('call', '50'),  ('fold', '')].
        """
        # Initial value to be returned
        fill_empty = [[("", "")]]

        # Filter relevant data of a specific stage
        hand = [x for x in hand if stage in x]
        if hand == []:
            return fill_empty

        # Get the first content of a played hand for a specific stage
        target = hand[0]

        # Pattern to extract
        regex = rf"{player}: (\w+) (\d+)?"

        # Apply regex
        result = re.findall(regex, target)

        # Guarantee that empty list will not be return
        if result == []:
            return fill_empty

        return [result]

    def get_allin(self, player: str, hand: list[str], stage: str) -> list[bool]:
        """
        Get if the player is all-in (it is boolean, True or False).

        Args:
            player (str): Name of the player.
            hand (list): List of texts from a specific hand.
            stage (str): Name of the stage to be considered

        Returns:
            list: List containing True or False, like [True].
        """
        # Filter relevant data of a specific stage
        hand = [x for x in hand if stage in x]

        # If a specific stage is not available, return False
        if hand == []:
            return [False]

        # Pattern to extract
        regex = rf"{player}: .* (all-in)"

        # Get the first content of a played hand for a specific stage
        target = hand[0]

        # Apply regex
        all_in_data = re.findall(regex, target)

        # Was some all-in action found? (True/False)
        result = all_in_data != []

        return [result]

    def get_showed_card(
        self, player: str, hand: list[str]
    ) -> list[list[str]] | list[list[None]]:
        """
        Get cards, if the player showed them.

        Args:
            player (str): Name of the player.
            hand (list): List of texts from a specific hand.

        Returns:
            list: List containing the cards of the player, like [['As', 'Ks']].
        """
        # Pattern to extract
        regex = rf"{player} .*(?:mucked|showed) \[(\S+) (\S+)\]"

        # Defauld value for empty values
        fill_empty = [[None, None]]

        # Get the last content of a played hand
        target = hand[-1]

        # Apply regex
        result = re.findall(regex, target)

        # Guarantee that empty list will not be return
        if result == []:
            return fill_empty

        return result

    def get_card_combination(self, player: str, hand: list[str]) -> list[str]:
        """
        Get cards combination, if the player went to showdown.

        Args:
            player (str): Name of the player.
            hand (list): List of texts from a specific hand.

        Returns:
            list: List containing the card combination, like ['a pair of Kings'].
        """
        # Pattern to extract
        regex = rf"{player}.*showed.*with (.*)"

        # Get the last content of a played hand
        target = hand[-1]

        # Apply regex
        result = re.findall(regex, target)

        # Normalize output
        result = self._guarantee_unicity(result, fill=None)

        return result

    def get_result(self, player: str, hand: list[str]) -> list[str]:
        """
        Get the result of a showdown for a player.

        Args:
            player (str): Name of the player.
            hand (list): List of texts from a specific hand.

        Returns:
            list: List containing the result, like ['won'].
        """
        # Pattern to extract
        regex = rf"Seat \d+: {player} .*(\bfolded\b|\bwon\b|\blost\b|\bmucked\b).*"

        # Get the last content of a played hand
        target = hand[-1]

        # Apply regex
        result = re.findall(regex, target)

        # Normalize output
        result = self._guarantee_unicity(result, fill="non-sd win")

        return result

    def get_balance(self, player: str, hand: list[str]) -> list[float | None]:
        """
        Get how much a player won from the pot.

        Args:
            player (str): Name of the player.
            hand (list): List of texts from a specific hand.

        Returns:
            list: List containing the balance, like ['320'].
        """
        # Pattern to extract
        regex = rf"Seat \d: {player}.*(?:collected|won) \((\d+)\)"

        # Get the last content of a played hand
        target = hand[-1]

        # Apply regex
        result = re.findall(regex, target)

        # Normalize output
        result = [
            float(x) if x is not None else None
            for x in self._guarantee_unicity(result, fill=None)
        ]

        return result

    def get_board(
        self, hand: list[str], stage: str
    ) -> (
        list[tuple[str, str, str]]  # Flop
        | list[tuple[str, str, str, str]]  # Turn
        | list[tuple[str, str, str, str, str]]  # River
        | list[tuple[()]]
    ):
        """
        Get the cards on the board.

        Args:
            hand (list): List of texts from a specific hand.
            stage (str): Name of the stage to be considered

        Returns:
            list: List containing the cards on the board, like [('As', 'Ks', '2h')].
        """
        # Defauld value for empty values
        fill_empty = [()]

        # Filter hands and reduce strings (to speed up the search)
        hand = [x[0:30] for x in hand if stage in x]

        # If no info for a specific stage is found, return empty values
        if hand == []:
            return fill_empty

        # Pattern to extract board, that is between [ and ]
        pre_regex = r"\[(.*)\]"

        # Pattern to extract cards on the board
        regex_cards = r"\b([\dTJQKA]{1}[shdc]{1})\b"

        # Get the first content of a played hand for a specific stage
        target = hand[0]

        # Apply regex
        pre_selection = re.findall(pre_regex, target)
        if pre_selection == []:
            return fill_empty
        else:
            result = re.findall(regex_cards, pre_selection[0])

        return [tuple(result)]

    def get_seat(self, player: str, hand: list[str]) -> list[int]:
        """
        Get the seat number of the player.

        Args:
            player (str): Name of the player.
            hand (list): List of texts from a specific hand.

        Returns:
            list: List containing the number of the seat, like ['1'].
        """
        # Pattern to extract
        regex = rf"\nSeat (\d+): {player}.*"

        # Get the first content of a played hand for a specific stage
        target = hand[0]

        # Apply regex
        result = re.findall(regex, target)

        # Normalize output
        result = [int(x) for x in self._guarantee_unicity(result, fill=None)]

        return result

    def get_position(self, player: str, hand: list[str]) -> list[str]:
        """
        Get position of the player (for example, 'button').

        Args:
            player (str): Name of the player.
            hand (list): List of texts from a specific hand.

        Returns:
            list: List containing the position of the player, like ['small blind'].
        """
        # Pattern to extract
        regex = rf"Seat \d+\: {player} \((button|small blind|big blind)\) "

        # Get the last content of a played hand
        target = hand[-1]

        # Apply regex
        result = re.findall(regex, target)

        # Normalize output
        result = self._guarantee_unicity(result, fill=None)

        return result

    def get_final_rank(self, player: str, hand: list[str]) -> list[int]:
        """
        Get final rank of the player (for example, '14').

        Args:
            player (str): Name of the player.
            hand (list): List of texts from a specific hand.

        Returns:
            list: List containing the final rank of the player, like ['1'].
        """
        # Filter content from SHOW_DOWN
        hand = [x for x in hand if "SHOW DOWN ***" in x]

        if hand != []:

            # Pattern to extract
            regex = rf"{player} finished the tournament in (\d+).*"

            # Get the first SHOW_DOWN content of a played hand
            target = hand[0]

            # Apply regex
            list_of_results = re.findall(regex, target)

            # Normalize
            list_of_int = [
                int(x) if x.replace(".", "").isdigit() else -1 for x in list_of_results
            ]

            # Try another pattern, if nothing is found
            if list_of_int == []:

                # Pattern to extract
                regex = rf"{player} (wins) the tournament"

                # Apply regex again
                result_of_regex = re.findall(regex, target)

                # Position 1, if wins
                if result_of_regex == ["wins"]:
                    return [1]
                else:
                    return [-1]
            else:
                # Get the maximum value and return in a list
                result = [max(list_of_int)]

                return result

        # If no SHOW_DOWN content is found, return -1
        else:
            return [-1]

    def get_prize(self, player: str, hand: list[str]) -> list[float] | list[None]:
        """
        Get prize of the player (for example, '$50.00').

        Args:
            player (str): Name of the player.
            hand (list): List of texts from a specific hand.

        Returns:
            list: List containing the prize, like ['$50.00'].
        """
        # Filter content from SHOW_DOWN
        hand = [x for x in hand if "SHOW DOWN ***" in x]

        if hand != []:

            # Pattern to extract
            regex = rf"{player} .* (?:and receives|and received) (?:[$€£]?)\s*(\d+(?:\.\d+)?)"

            # Get the first SHOW_DOWN content of a played hand
            target = hand[0]

            # Apply regex
            final_result = re.findall(regex, target)[:1]

            # Normalize output
            if final_result == []:
                return [None]
            else:
                return final_result

        else:
            return [None]



from pydantic import BaseModel
from datetime import datetime


class ValidateInput(BaseModel):
    Modality: list[str]
    TableSize: list[int]
    BuyIn: list[str]
    TournID: list[str]
    TableID: list[str]
    HandID: list[str]
    LocalTime: list[datetime]
    Level: list[str]
    Ante: list[float | None]
    Blinds: list[tuple[float, float]]
    Owner: list[str]
    OwnersHand: list[tuple[str, str]]
    Playing: list[int]
    Player: list[str]
    Seat: list[int]
    PostedAnte: list[float | None]
    Position: list[str | None]
    PostedBlind: list[float | None]
    Stack: list[float | None]
    PreflopAction: list[list[tuple[str, str]]]
    FlopAction: list[list[tuple[str, str]]]
    TurnAction: list[list[tuple[str, str]]]
    RiverAction: list[list[tuple[str, str]]]
    AnteAllIn: list[bool]
    PreflopAllIn: list[bool]
    FlopAllIn: list[bool]
    TurnAllIn: list[bool]
    RiverAllIn: list[bool]
    BoardFlop: list[tuple[str, str, str] | tuple[()]]
    BoardTurn: list[tuple[str, str, str, str] | tuple[()]]
    BoardRiver: list[tuple[str, str, str, str, str] | tuple[()]]
    ShowDown: list[tuple[str, str] | tuple[None, None]]
    CardCombination: list[str | None]
    Result: list[str]
    Balance: list[float | None]
    FinalRank: list[int]
    Prize: list[float | None]



import os
import sys
import datetime

from pokerdf.core.read_and_convert import execute_in_parallel


def main() -> None:
    """
    Main function to process command line arguments and execute the 'convert' command.

    - Checks if the command is 'convert'.
    - If 'convert', generates a session ID and creates a destination path.
    - Executes the pipeline function `execute_in_parallel` to process files from source to destination.

    Raises:
        SystemExit: If there are not enough arguments or if an invalid command is provided.
    """

    if len(sys.argv) < 3:
        print("Usage: pokerdf convert <path>")
        sys.exit(1)

    command = sys.argv[1]
    source_path = sys.argv[2]

    if command == "convert":

        # Check if the source path exists
        if not os.path.exists(source_path):
            print(f"The source path '{source_path}' does not exist.")
            sys.exit(1)
        # Check if the source path is a directory
        if not os.path.isdir(source_path):
            print(f"The source path '{source_path}' is not a directory.")
            sys.exit(1)
        # Check if the source path is empty
        if not os.listdir(source_path):
            print(f"The source path '{source_path}' is empty.")
            sys.exit(1)
        # Check if the source path is a valid poker hand history file
        if not any(file.endswith(".txt") for file in os.listdir(source_path)):
            print(
                f"The source path '{source_path}' does not contain any poker hand history files."
            )
            sys.exit(1)

        # Get start time
        start_time = datetime.datetime.now()

        # Generate session ID
        session_id = datetime.datetime.now().strftime("%Y%m%d-%H%M%S")

        # Generate destionation path
        destination_path = f"./output/{session_id}"

        # Create folder
        os.makedirs(destination_path)

        # Execute pipeline
        execute_in_parallel(source=source_path, destination=destination_path)

        # Get end time
        end_time = datetime.datetime.now()
        elapsed_time = end_time - start_time
        # Get the completed time in hours, minutes, and seconds
        hours, remainder = divmod(elapsed_time.total_seconds(), 3600)
        minutes, seconds = divmod(remainder, 60)
        # Print the completed time in a readable format
        print(
            f"Processing completed in {int(hours)} hours, {int(minutes)} minutes, and {int(seconds)} seconds."
        )

    else:
        print(f"The command '{command}' does not exist.")


if __name__ == "__main__":
    main()



from typing import Any
from pokerdf.regex.regex_patterns import RegexPatterns

r = RegexPatterns()


def capture_common_data(splitted_hand: list[str]) -> dict[str, Any]:
    """
    Captures the common data of the tournament

    Args:
        hand (list): List of texts from a specific hand.

    Returns:
        dict: Dictionary of captured values.
    """
    row: dict[str, Any] = {}
    row["Modality"] = r.get_modality(splitted_hand)
    row["TableSize"] = r.get_table_size(splitted_hand)
    row["BuyIn"] = r.get_buyin(splitted_hand)
    row["TournID"] = r.get_tourn_id(splitted_hand)
    row["Owner"] = r.get_owner(splitted_hand)

    return row


def capture_general_data_of_the_hand(splitted_hand: list[str]) -> dict[str, Any]:
    """
    Captures the general data of the hand

    Args:
        hand (list): List of texts from a specific hand.

    Returns:
        dict: Dictionary of captured values.
    """
    row: dict[str, Any] = {}
    row["HandID"] = r.get_hand_id(splitted_hand)
    row["TableID"] = r.get_table_id(splitted_hand)
    row["LocalTime"] = r.get_time(splitted_hand)
    row["Level"] = r.get_level(splitted_hand)
    row["Ante"] = r.get_ante(splitted_hand)
    row["Blinds"] = r.get_blinds(splitted_hand)
    row["OwnersHand"] = r.get_owner_cards(splitted_hand)
    row["Playing"] = r.get_number_of_active_players(splitted_hand)
    row["BoardFlop"] = r.get_board(splitted_hand, stage="FLOP ***")
    row["BoardTurn"] = r.get_board(splitted_hand, stage="TURN ***")
    row["BoardRiver"] = r.get_board(splitted_hand, stage="RIVER ***")

    return row


def capture_specific_data_of_the_player(
    splitted_hand: list[str], player: str
) -> dict[str, Any]:
    """
    Captures the specific data of a player

    Args:
        hand (list): List of texts from a specific hand.
        player (str): Name of the player.

    Returns:
        dict: Dictionary of captured values.
    """
    row: dict[str, Any] = {}
    row["Player"] = [player]
    row["Seat"] = r.get_seat(player, splitted_hand)
    row["PostedAnte"] = r.get_posted_ante(player, splitted_hand)
    row["Position"] = r.get_position(player, splitted_hand)
    row["PostedBlind"] = r.get_posted_blind(player, splitted_hand)
    row["Stack"] = r.get_stack(player, splitted_hand)
    row["PreflopAction"] = r.get_actions(player, splitted_hand, stage="HOLE CARDS ***")
    row["FlopAction"] = r.get_actions(player, splitted_hand, stage="FLOP ***")
    row["TurnAction"] = r.get_actions(player, splitted_hand, stage="TURN ***")
    row["RiverAction"] = r.get_actions(player, splitted_hand, stage="RIVER ***")
    row["AnteAllIn"] = r.get_allin(player, splitted_hand, stage=" posts the ante ")
    row["PreflopAllIn"] = r.get_allin(player, splitted_hand, stage="HOLE CARDS ***")
    row["FlopAllIn"] = r.get_allin(player, splitted_hand, stage="FLOP ***")
    row["TurnAllIn"] = r.get_allin(player, splitted_hand, stage="TURN ***")
    row["RiverAllIn"] = r.get_allin(player, splitted_hand, stage="RIVER ***")
    row["ShowDown"] = r.get_showed_card(player, splitted_hand)
    row["CardCombination"] = r.get_card_combination(player, splitted_hand)
    row["Result"] = r.get_result(player, splitted_hand)
    row["Balance"] = r.get_balance(player, splitted_hand)
    row["FinalRank"] = r.get_final_rank(player, splitted_hand)
    row["Prize"] = r.get_prize(player, splitted_hand)

    return row



import os
import pandas as pd
from typing import List
from joblib import Parallel, delayed
from pokerdf.validation.pydantic_modules import ValidateInput
from pokerdf.utils.strings import PLATFORM
from pokerdf.regex.regex_execution import (
    capture_common_data,
    capture_general_data_of_the_hand,
    capture_specific_data_of_the_player,
    r,
)
import warnings

warnings.simplefilter(action="ignore", category=FutureWarning)


def get_files_paths(path: str) -> List[str]:
    """
    Retrieve the paths of relevant files in the specified directory.

    Args:
        path (str): The directory path to search for files.

    Returns:
        List[str]: A list of file paths that match the criteria.
    """
    # Get files names
    list_of_all_files_names = os.listdir(path)

    # Order by ID
    list_of_all_files_names.sort()

    # Keep relevant files only
    list_of_selected_files = [
        file_name
        for file_name in list_of_all_files_names
        if file_name.startswith("HH") and file_name.endswith(".txt")
    ]

    # Compose the final path for each file
    paths = [os.path.join(path, file_name) for file_name in list_of_selected_files]

    return paths


def compose_dataframe() -> pd.DataFrame:
    """
    Create an empty DataFrame with predefined columns to hold poker data.

    Returns:
        pd.DataFrame: An empty DataFrame with predefined columns.
    """
    # Compose default dataframe
    df = pd.DataFrame(
        {
            "Modality": pd.Series(dtype="object"),
            "TableSize": pd.Series(dtype="int64"),
            "BuyIn": pd.Series(dtype="object"),
            "TournID": pd.Series(dtype="object"),
            "TableID": pd.Series(dtype="object"),
            "HandID": pd.Series(dtype="object"),
            "LocalTime": pd.Series(dtype="datetime64[ns]"),
            "Level": pd.Series(dtype="object"),
            "Ante": pd.Series(dtype="float64"),
            "Blinds": pd.Series(dtype="object"),
            "Owner": pd.Series(dtype="object"),
            "OwnersHand": pd.Series(dtype="object"),
            "Playing": pd.Series(dtype="int64"),
            "Player": pd.Series(dtype="object"),
            "Seat": pd.Series(dtype="int64"),
            "PostedAnte": pd.Series(dtype="float64"),
            "Position": pd.Series(dtype="object"),
            "PostedBlind": pd.Series(dtype="float64"),
            "Stack": pd.Series(dtype="float64"),
            "PreflopAction": pd.Series(dtype="object"),
            "FlopAction": pd.Series(dtype="object"),
            "TurnAction": pd.Series(dtype="object"),
            "RiverAction": pd.Series(dtype="object"),
            "AnteAllIn": pd.Series(dtype="bool"),
            "PreflopAllIn": pd.Series(dtype="bool"),
            "FlopAllIn": pd.Series(dtype="bool"),
            "TurnAllIn": pd.Series(dtype="bool"),
            "RiverAllIn": pd.Series(dtype="bool"),
            "BoardFlop": pd.Series(dtype="object"),
            "BoardTurn": pd.Series(dtype="object"),
            "BoardRiver": pd.Series(dtype="object"),
            "ShowDown": pd.Series(dtype="object"),
            "CardCombination": pd.Series(dtype="object"),
            "Result": pd.Series(dtype="object"),
            "Balance": pd.Series(dtype="float64"),
            "FinalRank": pd.Series(dtype="int64"),
            "Prize": pd.Series(dtype="float64"),
        }
    )
    return df


def apply_regex(txt: str) -> pd.DataFrame:
    """
    Apply regex functions to parse the hand history text and collect relevant data.

    Args:
        txt (str): The text content of the poker hand history file.

    Returns:
        pd.DataFrame: A DataFrame containing the parsed data from the hand history.
    """
    # Generate dataframe
    df = compose_dataframe()

    # Spliting tournament's hands in a list
    list_of_hands_as_text = txt.split(f"{PLATFORM} ")

    # Cleaning list_of_hands_as_text
    string_to_remove = "\ufeff"
    if string_to_remove in list_of_hands_as_text:
        list_of_hands_as_text.remove(string_to_remove)
    list_of_hands_as_text = [hand for hand in list_of_hands_as_text if hand is not None]
    list_of_hands_as_text = [hand for hand in list_of_hands_as_text if len(hand) > 0]

    # Capture common info about the tournament
    common = capture_common_data(list_of_hands_as_text[0].split("\n*** "))

    for hand in list_of_hands_as_text:

        # Split hand in stages (pre-flop/flop/turn/river)
        splited_hand = hand.split("\n*** ")

        # Capture general info of the hand
        general = capture_general_data_of_the_hand(splited_hand)

        # Get players
        players = r.get_players(splited_hand)

        # Iterate over players
        for player in players:

            # Capture specific info of players' actions
            specific = capture_specific_data_of_the_player(splited_hand, player)

            # Combine collected info
            collected_data = {**common, **general, **specific}

            # Validate
            ValidateInput(**collected_data)

            # Convert to dataframe
            result = pd.DataFrame(collected_data)

            # Concat to the final results
            df = pd.concat([df, result])

    return df


def convert_txt_to_tabular_data(path: str) -> pd.DataFrame:
    """
    Convert a poker hand history text file into a structured DataFrame.

    Args:
        path (str): The path to the .txt file containing the hand history.

    Returns:
        pd.DataFrame: A DataFrame with parsed data from the hand history.
    """
    with open(path, "r", encoding="utf-8", errors="replace") as file:
        txt = file.read()
        result = apply_regex(txt)

    return result


def _save_log(msg: str, destination: str, file_name: str) -> None:
    """
    Save a log message to a file.

    Args:
        msg (str): The message to be logged.
        destination (str): The folder where the log file will be saved.
        file_name (str): The name of the log file.
    """
    # Compose path of the log
    path = os.path.join(destination, file_name)

    # Open the file
    file = open(path, "a")

    # Write content
    file.write(msg + "\n")

    # Close the writing process
    file.close()


class DataProcessing:
    """
    Process and save a poker hand history file, logging the result.

    Args:
        path (str): The path to the hand history file.
        destination (str): The directory where the processed data will be saved.
    """

    def __init__(self, path: str, destination: str) -> None:
        self.path = path
        self.destination = destination

    def run(self) -> None:
        """
        Trigger the data processing.
        """
        try:

            # Convert text to pd.DataFrame
            df = convert_txt_to_tabular_data(self.path).reset_index(drop=True)

            # Compose name of the .parquet file (the Tournament ID + the Local Time)
            clean_datetime = str(df.LocalTime[0]).replace("-", "")[:8]
            file_name = clean_datetime + "-T" + str(df.TournID[0]) + ".parquet"

            # Path to save the file
            destination_path = os.path.join(self.destination, file_name)

            # Save the table
            df.to_parquet(destination_path, index=False)

            # Log / print DONE status
            msg = "   DONE: " + self.path.split("/")[-1]
            _save_log(msg, self.destination, "success.txt")
            print(msg)

        except Exception as e:

            # Log / print FAIL status
            msg = "   FAIL: " + self.path.split("/")[-1]
            msg += " (" + str(e) + ")"
            _save_log(msg, self.destination, "fail.txt")
            print(msg)


def execute_in_parallel(source: str, destination: str) -> None:
    """
    Function to run the DataProcessing with multiple cores
    """

    # Get all paths
    all_paths = get_files_paths(source)

    # Run a DataProcessing in parallel.
    Parallel(n_jobs=-1)(
        delayed(DataProcessing(path, destination).run)() for path in all_paths
    )
